"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SlackService = void 0;
const tslib_1 = require("tslib");
const axios_1 = require("axios");
const common_1 = require("@nestjs/common");
const constants_1 = require("./constants");
const utils_1 = require("./utils");
let SlackService = class SlackService {
    constructor(options, client, log, webhookUrl) {
        this.options = options;
        this.client = client;
        this.log = log;
        this.webhookUrl = webhookUrl;
    }
    sendText(text, opts) {
        return this.postMessage(Object.assign({ text }, opts));
    }
    sendBlocks(blocks, opts) {
        return this.postMessage(Object.assign({ blocks }, opts));
    }
    postMessage(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const requestTypes = {
                api: () => tslib_1.__awaiter(this, void 0, void 0, function* () { return this.runApiRequest(req); }),
                webhook: () => tslib_1.__awaiter(this, void 0, void 0, function* () { return this.runWebhookRequest(req); }),
                stdout: () => tslib_1.__awaiter(this, void 0, void 0, function* () { return this.runStdoutRequest(req); }),
                google: () => tslib_1.__awaiter(this, void 0, void 0, function* () { return this.runGoogleLoggingRequest(req); }),
            };
            yield requestTypes[this.options.type]();
        });
    }
    runApiRequest(req) {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            (0, utils_1.invariant)(this.options.type === 'api');
            (0, utils_1.invariant)(this.client, 'expected this.client to be WebClient');
            const channel = (_a = req.channel) !== null && _a !== void 0 ? _a : this.options.defaultChannel;
            (0, utils_1.invariant)(channel, 'neither channel nor defaultChannel was applied');
            yield this.client.chat.postMessage(Object.assign({ channel }, req));
        });
    }
    runWebhookRequest(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            (0, utils_1.invariant)(this.options.type === 'webhook');
            (0, utils_1.invariant)(this.webhookUrl, 'expected webhook url to exist');
            yield axios_1.default.post(this.webhookUrl, req).catch((error) => {
                (0, utils_1.invariant)(error.response);
                throw new Error(`Could not send request to Slack Webhook: ${error.response.data}`);
            });
        });
    }
    runStdoutRequest(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            (0, utils_1.invariant)(this.options.type === 'stdout');
            this.options.output(req);
        });
    }
    runGoogleLoggingRequest(req) {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            (0, utils_1.invariant)(this.options.type === 'google');
            const metadata = {
                severity: 'NOTICE',
                'logging.googleapis.com/labels': { type: 'nestjs-slack' },
                'logging.googleapis.com/operation': {
                    producer: 'github.com/bjerkio/nestjs-slack@v1',
                },
            };
            (0, utils_1.invariant)(this.log, 'expected Google Logger instance');
            const channel = (_a = req.channel) !== null && _a !== void 0 ? _a : this.options.defaultChannel;
            yield this.log.write(this.log.entry(metadata, {
                slack: Object.assign({ channel }, req),
            }));
        });
    }
};
SlackService = tslib_1.__decorate([
    (0, common_1.Injectable)(),
    tslib_1.__param(0, (0, common_1.Inject)(constants_1.SLACK_MODULE_OPTIONS)),
    tslib_1.__param(1, (0, common_1.Inject)(constants_1.SLACK_WEB_CLIENT)),
    tslib_1.__param(2, (0, common_1.Inject)(constants_1.GOOGLE_LOGGING)),
    tslib_1.__param(3, (0, common_1.Inject)(constants_1.SLACK_WEBHOOK_URL)),
    tslib_1.__metadata("design:paramtypes", [Object, Function, Function, String])
], SlackService);
exports.SlackService = SlackService;
